import py_cui

from time import sleep
from queue import Queue
from shlex import quote
from re import findall

from recoverpy import views_handler as VIEWS_HANDLER
from recoverpy import menu_with_block_display as BLOCK_DISPLAY_MENU
from recoverpy import saver as SAVER
from recoverpy import search_functions as SEARCH
from recoverpy import logger as LOGGER


class SearchView(BLOCK_DISPLAY_MENU.MenuWithBlockDisplay):
    """Search view is displayed after Parameters view.
    On the left hand scroll view, results from the grep command will be listed.
    On the right hand textbox, result of a dd command will be displayed when the user
    selects a block.

    Args:
        BLOCK_DISPLAY_MENU (MenuWithBlockDisplay: Composition to inherit block display
        methods.

    Attributes:
        master (py_cui.PyCUI): PyCUI constructor.
        queue_object (Queue): Queue object where grep command stdout will be stored.
        result_index (int): Number of results already processed.
        grep_progress (str): Formated output of 'progress' command.
        partition (str): System partition selected by user for search.
        block_size (int): Size of partition block for dd parsing.
        searched_string (str): String given by the user that will be searched by dd command.
    """

    def __init__(self, master: py_cui.PyCUI, partition: str, string_to_search: str):
        """Constructor for Search view

        Args:
            master (py_cui.PyCUI): PyCUI constructor
            partition (str): System partition to search
        """

        super().__init__()

        self.master = master
        self.master.set_refresh_timeout(1)

        self.queue_object = Queue()
        self.result_index = 0

        self.grep_progress = ""

        self.partition = partition
        self.block_size = 512

        self.searched_string = string_to_search

        LOGGER.write("info", "Starting 'SearchView' CUI window")

        self.create_ui_content()

        SEARCH.start_search(self)

        LOGGER.write(
            "info",
            f"Raw searched string:\n{self.searched_string}",
        )
        LOGGER.write(
            "info",
            f"Formated searched string:\n{quote(self.searched_string)}",
        )

    def set_title(self):
        if self.grep_progress != "":
            title = f"{self.grep_progress} - {str(self.result_index)} results"
        else:
            title = f"{str(self.result_index)} results"

        self.master.set_title(title)

    def create_ui_content(self):
        """Handles the creation of the UI elements."""

        self.search_results_scroll_menu = self.master.add_scroll_menu(
            "Search results:", 0, 0, row_span=10, column_span=5, padx=1, pady=0
        )
        self.search_results_scroll_menu.add_key_command(py_cui.keys.KEY_ENTER, self.display_selected_block)

        self.result_content_box = self.master.add_text_block(
            "Block content:", 0, 5, row_span=9, column_span=5, padx=1, pady=0
        )
        self.result_content_box.add_key_command(py_cui.keys.KEY_F5, self.open_save_popup)
        self.result_content_box.add_key_command(py_cui.keys.KEY_F6, self.display_previous_block)
        self.result_content_box.add_key_command(py_cui.keys.KEY_F7, self.display_next_block)

        self.previous_button = self.master.add_button(
            "<",
            9,
            5,
            row_span=1,
            column_span=1,
            padx=1,
            pady=0,
            command=self.display_previous_block,
        )

        self.next_button = self.master.add_button(
            ">",
            9,
            8,
            row_span=1,
            column_span=1,
            padx=1,
            pady=0,
            command=self.display_next_block,
        )

        self.save_file_button = self.master.add_button(
            "Save Block",
            9,
            6,
            row_span=1,
            column_span=2,
            padx=1,
            pady=0,
            command=self.open_save_popup,
        )

        self.exit_button = self.master.add_button(
            "Exit",
            9,
            9,
            row_span=1,
            column_span=1,
            padx=1,
            pady=0,
            command=self.master.stop,
        )

    def populate_result_list(self):
        """Polls grep output and populate result list."""

        while True:
            try:
                new_results, self.result_index = SEARCH.yield_new_results(self.queue_object, self.result_index)
            except TypeError:
                # If no new results
                sleep(1)
                continue

            self.add_results_to_list(new_results=new_results)
            self.set_title()

            # Sleeps to avoid unnecessary overload
            sleep(1)

    def add_results_to_list(self, new_results: list):
        """Adds new results from the grep command to the left hand result list.

        Args:
            new_results (list): New results from the grep command.
        """

        for result in new_results:
            string_result = str(result)[2:-1]
            self.search_results_scroll_menu.add_item(string_result)

            LOGGER.write("debug", "New result found: " + string_result[:30] + " ...")

    def update_block_number(self):
        """Updates currently viewed block number when the user selects one in the list."""

        item = self.search_results_scroll_menu.get()
        inode = int(findall(r"^([0-9]+)\:", item)[0])
        self.current_block = str(int(inode / self.block_size))

        LOGGER.write(
            "debug",
            f"Displayed block set to {str(self.current_block)}",
        )

    def display_selected_block(self):
        """Function called when the user select a result in the left hand list.
        Bundles updating + displaying block.
        """

        self.update_block_number()
        self.display_block(self.current_block)

    def open_save_popup(self):
        """Opens a popup displaying save options."""

        view_choices = [
            "Save currently displayed block",
            "Explore neighboring blocks and save it all",
            "Cancel",
        ]
        self.master.show_menu_popup("How do you want to save it ?", view_choices, self.handle_save_popup_choice)

    def handle_save_popup_choice(self, choice: str):
        """Depending on user choice, function will either directly save the output in
        a text file, open a more detailed view called ResultsView or just exit.

        Args:
            choice (str): User choice given by open_save_popup function.
        """

        if choice == "Save currently displayed block":
            SAVER.save_result(current_block=self.current_block, result=self.current_result)

            self.master.show_message_popup("", "Result saved.")
        elif choice == "Explore neighboring blocks and save it all":
            VIEWS_HANDLER.open_view_results(partition=self.partition, block=self.current_block)
        elif choice == "Cancel":
            pass



from datetime import datetime

import logging

_LOG_FILE_PATH = None
_LOG_ENABLED = False
_LOGGER = None


def enable_logging():
    """Enable logging based on config file."""

    global _LOG_ENABLED

    _LOG_ENABLED = True


def disable_logging():
    """Disable logging based on config file."""

    global _LOG_ENABLED

    _LOG_ENABLED = False


def set_log_file_path(path):
    """Sets result save path based on config file"""

    global _LOG_FILE_PATH

    _LOG_FILE_PATH = path
    start_logging()


def start_logging():
    """Main logging function, initiates and configures the logger object."""

    global _LOG_FILE_PATH
    global _LOGGER

    time = datetime.now().strftime("%Y-%m-%d-%H%M%S")

    log_file_name = f"{_LOG_FILE_PATH}recoverpy-{time}.log"

    _LOGGER = logging.getLogger("main")
    _LOGGER.setLevel(logging.DEBUG)

    file_handler = logging.FileHandler(log_file_name)
    file_handler.setLevel(logging.DEBUG)

    _LOGGER.addHandler(file_handler)


def write(log_type, text: str):
    """Uses logger object to write to file."""

    global _LOG_ENABLED
    global _LOGGER

    if not _LOG_ENABLED:
        return

    if log_type == "debug":
        _LOGGER.debug(text)
    elif log_type == "info":
        _LOGGER.info(text)
    elif log_type == "warning":
        _LOGGER.warning(text)
    elif log_type == "error":
        _LOGGER.error(text)
    elif log_type == "critical":
        _LOGGER.critical(text)



from subprocess import check_output

from recoverpy import logger as LOGGER


class MenuWithBlockDisplay:
    """Composition aimed class for menus needing methods to display partition
    blocks content.

    Attributes:
        master (py_cui.PyCUI): PyCUI constructor.
        horizontal_char_limit (int): Number of chars that can fit horizontally in
            the left hand result box.
        current_block (int): Partition block number currently displayed.
        current_result (int): Block text content found with dd command.
        result_content_box (PyCUI.TextBox): TextBox in which dd command result
            will be displayed.
        partition (str): System partition selected by user for search.
    """

    def __init__(self):
        """Constructor for MenuWithBlockDisplay"""

        self.master = None

        self.horizontal_char_limit = 0

        self.current_block = 0
        self.current_result = None

        self.result_content_box = None
        self.partition = None

    def get_dd_result(self, block: str = None):
        """Stores a 'dd' command result in current_result var

        Args:
            block (str, optional): Partition block number. Defaults to None.
        """

        if block is None:
            block = self.current_block

        LOGGER.write(
            "debug",
            f"Getting 'dd' output for block {str(self.current_block)}",
        )

        try:
            dd_result = check_output(
                [
                    "dd",
                    f"if={self.partition}",
                    "count=1",
                    "status=none",
                    f"skip={block}",
                ]
            )
            # Try/Catch to decode raw result in utf-8
            try:
                self.current_result = dd_result.decode("utf-8")
            except:
                self.current_result = str(dd_result)
            self.current_block = block

            LOGGER.write("debug", "dd command successful")
        except:
            self.master.show_error_popup(
                "ERROR",
                f"Error while opening block {str(self.current_block)}",
            )
            LOGGER.write(
                "error",
                f"Error while opening block {str(self.current_block)}",
            )

    def update_textbox(self):
        """Formats 'dd' result by breaking lines with char_limit var.
        Then displays it in lefthand texbox.
        """
        self.update_char_limit()

        # Format raw result to display it in the text box
        result_lines = [
            str(self.current_result)[i : i + self.horizontal_char_limit]
            for i in range(0, len(str(self.current_result)), self.horizontal_char_limit)
        ]
        formated_result = "\n".join(result_lines)

        self.result_content_box.set_text(formated_result)
        self.result_content_box.set_title(f"Block {str(self.current_block)}")

        LOGGER.write(
            "debug",
            f"Textbox updated with block {str(self.current_block)}",
        )

    def display_previous_block(self):
        """Function to display block n-1 in textbox."""

        try:
            self.display_block(str(int(self.current_block) - 1))
        except ValueError:
            LOGGER.write(
                "error",
                f"Cannot display block {str(self.current_block)} - 1",
            )
            return

    def display_next_block(self):
        """Function to display block n+1 in textbox."""

        try:
            self.display_block(str(int(self.current_block) + 1))
        except ValueError:
            LOGGER.write(
                "error",
                f"Cannot display block {str(self.current_block)} + 1",
            )
            return

    def display_block(self, block: str):
        """Function to display given block number in textbox.

        Args:
            block (str): Partition block number.
        """

        self.get_dd_result(block=block)
        self.update_textbox()

    def update_char_limit(self):
        """Update horizontal character limit for textbox depending on terminal size."""

        text_box_dimensions = self.result_content_box.get_cursor_limits_horizontal()
        self.horizontal_char_limit = text_box_dimensions[1] - text_box_dimensions[0]
        LOGGER.write(
            "debug",
            f"Textbox char limit set to {self.horizontal_char_limit}",
        )



import py_cui

from re import findall

from recoverpy import views_handler as VIEWS_HANDLER
from recoverpy import helper as HELPER
from recoverpy import logger as LOGGER


class ParametersView:
    """Parameters view is the first window displayed.
    User is prompted to select a partition and a string to search in it.

    Attributes:
        partition_to_search (str): Partition selected by user.
        string_to_search (str): String entered by user.
        partitions_dict (dict): Dictionnary of system partitions found with
            lsblk command and their attributes.
    """

    def __init__(self, master: py_cui.PyCUI):
        """Constructor for Parameters view

        Args:
            master (py_cui.PyCUI): PyCUI constructor
        """

        self.master = master

        self.partition_to_search = None
        self.string_to_search = None
        self.partitions_dict = None

        LOGGER.write("info", "Starting 'ParametersView' CUI window")
        HELPER.is_user_root(window=self.master)

        self.create_ui_content()
        self.get_system_partitions()
        self.add_partitions_to_list()

    def create_ui_content(self):
        """Handles the creation of the UI elements."""

        self.partitions_list_scroll_menu = self.master.add_scroll_menu(
            "Select a partition to search:", 0, 0, row_span=9, column_span=5
        )
        self.partitions_list_scroll_menu.add_key_command(py_cui.keys.KEY_ENTER, self.select_partition)

        # Color rules
        self.partitions_list_scroll_menu.add_text_color_rule("Mounted at", py_cui.YELLOW_ON_BLACK, "contains")
        self.partitions_list_scroll_menu.set_selected_color(py_cui.GREEN_ON_BLACK)

        self.string_text_box = self.master.add_text_block("Enter a text to search:", 0, 5, row_span=9, column_span=5)

        self.confirm_search_button = self.master.add_button(
            "Start search",
            9,
            4,
            row_span=1,
            column_span=2,
            padx=0,
            pady=0,
            command=self.confirm_search,
        )

    def get_system_partitions(self):
        """Calls lsblk and lsblk output formatting."""

        partitions_list = HELPER.lsblk()
        self.partitions_dict = HELPER.format_partitions_list(window=self.master, raw_lsblk=partitions_list)

    def add_partitions_to_list(self):
        """Populates the partition list with lsblk output."""

        if self.partitions_dict is None:
            return

        for partition in self.partitions_dict:
            if self.partitions_dict[partition]["IS_MOUNTED"]:
                self.partitions_list_scroll_menu.add_item(
                    "Name: {name}  -  Type: {fstype}  -  Mounted at: {mountpoint}".format(
                        name=partition,
                        fstype=self.partitions_dict[partition]["FSTYPE"],
                        mountpoint=self.partitions_dict[partition]["MOUNT_POINT"],
                    )
                )
            else:
                self.partitions_list_scroll_menu.add_item(
                    "Name: {name}  -  Type: {fstype}".format(
                        name=partition, fstype=self.partitions_dict[partition]["FSTYPE"]
                    )
                )

            LOGGER.write(
                "debug",
                f"Partition added to list: {str(partition)}",
            )

    def select_partition(self):
        """Handles the user selection of a partition in the list."""

        selected_partition = findall(r"Name\:\ ([^\ \n]+)\ ", self.partitions_list_scroll_menu.get())[0]

        if self.partitions_dict[selected_partition]["IS_MOUNTED"]:
            # Warn the user to unmount his partition first
            self.master.show_warning_popup(
                "Warning",
                f"It is highly recommended to unmount {selected_partition} first.",
            )
        else:
            self.master.show_message_popup("", f"Partition {selected_partition} selected.")

        self.partition_to_search = "/dev/" + selected_partition.strip()

        LOGGER.write(
            "info",
            f"Partition selected: {self.partition_to_search}",
        )

    def confirm_search(self):
        """Checks if partition is selected and string is given.
        If all required elements are present, launch confirm_search function.
        """

        if not HELPER.is_user_root(window=self.master):
            return

        self.string_to_search = self.string_text_box.get()

        LOGGER.write("info", "Starting search")

        if self.partition_to_search == "":
            # No partition selected
            self.master.show_message_popup("Error", "You have to select a partition to search.")
            LOGGER.write("warning", "No partition selected for search")
        elif self.string_to_search.replace(" ", "").replace("\n", "").replace("\t", "") == "":
            # Blank string to search
            self.master.show_message_popup("Error", "You have to enter a text to search.")
            LOGGER.write("warning", "No string given for search")
        else:
            # Prompt to confirm string
            self.master.show_yes_no_popup(
                "Do you want to start searching this text on partition {partition} ?".format(
                    partition=self.partition_to_search
                ),
                self.start_search,
            )

    def start_search(self, is_confirmed: bool):
        """Closes parameters view and open search view if confirmed.

        Args:
            is_confirmed (bool): User popup selection
        """

        if is_confirmed:
            VIEWS_HANDLER.close_view_parameters()
            VIEWS_HANDLER.open_view_search(
                partition=self.partition_to_search,
                string_to_search=self.string_to_search.strip(),
            )



import py_cui

from recoverpy import view_parameters as PARAMETERS_VIEW
from recoverpy import view_search as SEARCH_VIEW
from recoverpy import view_results as RESULTS_VIEW


_PARAMETERS_VIEW = None
_SEARCH_VIEW = None
_RESULTS_VIEW = None


def open_view_parameters():
    """Starts a ParametersView instance."""
    global _PARAMETERS_VIEW

    _PARAMETERS_VIEW = py_cui.PyCUI(10, 10)
    _PARAMETERS_VIEW.toggle_unicode_borders()
    _PARAMETERS_VIEW.set_title("Retrieve deleted or overwritten text files")
    PARAMETERS_VIEW.ParametersView(_PARAMETERS_VIEW)
    _PARAMETERS_VIEW.start()


def close_view_parameters():
    """Stops the global ParametersView instance."""
    global _PARAMETERS_VIEW

    _PARAMETERS_VIEW.stop()


def open_view_search(partition: str, string_to_search: str):
    """Starts a SearchView instance."""
    global _SEARCH_VIEW

    _SEARCH_VIEW = py_cui.PyCUI(10, 10)
    _SEARCH_VIEW.toggle_unicode_borders()
    _SEARCH_VIEW.set_title("View and explore found files")
    SEARCH_VIEW.SearchView(_SEARCH_VIEW, partition=partition, string_to_search=string_to_search)
    _SEARCH_VIEW.start()


def close_view_search():
    """Stops the global SearchView instance."""
    global _SEARCH_VIEW

    _SEARCH_VIEW.stop()


def open_view_results(partition: str, block: str):
    global _RESULTS_VIEW

    _RESULTS_VIEW = py_cui.PyCUI(10, 10)
    _RESULTS_VIEW.toggle_unicode_borders()
    _RESULTS_VIEW.set_title("")
    RESULTS_VIEW.ResultsView(_RESULTS_VIEW, partition=partition, initial_block=block)
    _RESULTS_VIEW.start()



from queue import Queue
from threading import Thread

import io
import re
import time

from subprocess import PIPE, DEVNULL, Popen, check_output
from recoverpy import helper as HELPER
from recoverpy import logger as LOGGER


def monitor_progress(search_view, grep_pid: int):
    """Uses 'progress' tool to monitor grep advancement.

    Args:
        search_view (SearchView): Current PyCUI search view
        grep_pid (int): PID of grep process
    """

    while True:
        output = check_output(["progress", "-p", str(grep_pid)], stderr=DEVNULL).decode("utf8")

        try:
            progress = re.findall(r"([0-9]+\.[0-9]+\%[^\)]+\))", output)[0]
        except IndexError:
            continue

        search_view.grep_progress = progress
        LOGGER.write("debug", f"Progress: {progress}")
        search_view.set_title()
        time.sleep(1)


def start_search(search_view):
    """Function is called within __init__
    Launches:
    - Process executing the grep command.
    - If available, thread using 'progress' tool to monitor grep.
    - Thread to store the grep output in a queue object.
    - Thread to populate the result box dynamically.

    Args:
        search_view (SearchView): Current PyCUI search view
    """

    grep_process = create_grep_process(searched_string=search_view.searched_string, partition=search_view.partition)

    if HELPER.is_progress_installed():
        monitor_progress_thread = Thread(
            target=monitor_progress,
            args=(
                search_view,
                grep_process.pid,
            ),
        )
        monitor_progress_thread.daemon = True
        monitor_progress_thread.start()

    enqueue_grep_output_thread = Thread(
        target=enqueue_grep_output,
        args=(grep_process.stdout, search_view.queue_object),
    )
    enqueue_grep_output_thread.daemon = True
    enqueue_grep_output_thread.start()

    LOGGER.write("debug", "Started searching thread")

    yield_results_thread = Thread(target=search_view.populate_result_list)
    yield_results_thread.daemon = True
    yield_results_thread.start()

    LOGGER.write("debug", "Started output fetching thread")


def create_grep_process(searched_string: str, partition: str) -> Popen:
    """Instantiate a process executing the grep search.

    Args:
        searched_string (str): String to search
        partition (str): Partition to search into

    Returns:
        Popen: Created process
    """

    grep_process = Popen(
        ["grep", "-a", "-b", searched_string, partition],
        stdin=None,
        stdout=PIPE,
        stderr=None,
    )

    return grep_process


def enqueue_grep_output(out: io.BufferedReader, queue: Queue):
    """Function called in a thread to store the grep command output in
    a queue object.

    Args:
        out (io.BufferedReader): Output of grep process
        queue (Queue): Queue object to store stdout
    """

    for line in iter(out.readline, b""):
        queue.put(line)
    out.close()


def yield_new_results(queue_object: Queue, result_index: int) -> tuple:
    """Probes the queue object for new results.
    If any, returns it to populate the result box.

    Args:
        queue_object (Queue): Queue object storing grep stdout
        result_index (int): [Current result list index

    Returns:
        tuple: Tuple with 1. List of new results 2. New result index
    """

    new_results = []

    # Returns if no new result
    if len(list(queue_object.queue)) == result_index:
        return None

    queue_list = list(queue_object.queue)

    for queue_element in queue_list[result_index:]:
        new_results.append(queue_element)

    result_index = len(queue_list)

    return new_results, result_index



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['recoverpy']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.4.1,<6.0.0', 'py-cui>=0.1.3,<0.2.0']

setup_kwargs = {
    'name': 'recoverpy',
    'version': '1.2.2',
    'description': 'A CUI to recover overwritten or deleted data.',
    'long_description': '# Recoverpy [![GitHub release (latest by date)](https://img.shields.io/github/v/release/pablolec/recoverpy)](https://github.com/PabloLec/recoverpy/releases/) [![GitHub](https://img.shields.io/github/license/pablolec/recoverpy)](https://github.com/PabloLec/recoverpy/blob/main/LICENCE) [![Downloads](https://static.pepy.tech/personalized-badge/recoverpy?period=total&units=abbreviation&left_color=grey&right_color=red&left_text=Downloads)](https://pepy.tech/project/recoverpy) [![Tests](https://github.com/PabloLec/recoverpy/actions/workflows/recoverpy-tests.yml/badge.svg?branch=main)](#)\n\nRecoverpy is a CUI utility written in Python with the help of [py_cui](https://github.com/jwlodek/py_cui "py_cui").\n\nYou can already find plenty of solutions to recover deleted files but it can be a hassle to recover overwritten files. Recoverpy search through every inodes to find your request.\n\n## Demo\n\n<p align="center">\n    <img src="docs/assets/demo.gif">\n</p>\n\n## Installation\n\n:penguin: The main prerequisite is having a Linux system.\n\n**Mandatory:** To list and search through your partitions, recoverpy uses `grep`, `dd`, and `lsblk` commands.  \n**Optional:** To display real time grep progress, you can install `progress` tool.\n\nTo install all dependencies:\n- Debian-like: `apt install grep coreutils util-linux progress`  \n- Arch: `pacman -S grep coreutils util-linux progress`  \n- Fedora: `dnf install grep coreutils util-linux progress`  \n\n**Installation from pip**: `python3 -m pip install recoverpy`  \n\n*Pip should be already installed if you have Python >=3.4. Otherwise, see [\npip docs](https://pip.pypa.io/en/stable/installing/ "pip docs") for installation.*\n\nTo update: `python3 -m pip install --upgrade recoverpy`\n\n## Usage\n\n```bash\npython3 -m recoverpy\n```\n\n**You must have root access to use recoverpy**.\n\nIf you are not logged as root use `sudo recoverpy` or log in with `su -` before execution.\n\nFirst, **select the system partition** in which your file was. If you are out of luck, you can alternatively search in your home partition, maybe your IDE, text editor, etc. made a backup at some point.\n\nThen, **type a text to search**. You can now start the search.\n\nNote that searching a string in a whole partition may take _a while_. (see [euphemism](https://en.wikipedia.org/wiki/Euphemism "euphemism"))\n\nResults will appear in the left-hand box. Select a result to display the corresponding partition block content in the right-hand box.\n\nOnce you have found your precious, select `Save`.\nYou can now either save this block individually or explore neighboring blocks for the remaining parts of the file. You could then save it all in one file.\n\n**Save path is set in `conf.yaml`. Default is `/tmp/`.**\n\n## Tips\n\n- Always do backups! Yes, maybe too late...\n- **Unmount your partition before you do anything!** Although you can search with your partition still mounted, it is highly recommended to unmount your partition to avoid any alteration to your file.\n\nRegarding the string you search:\n\n- Be concise, find something that could be unique to your file.\n- Stay simple, your string is escaped but exotic characters may affect your results.\n- Try to remember the last edit you have made to your file.\n\nWhen you have a match:\n\n- Use the option to explore neighboring blocks to make sure you do not miss some part of your file.\n',
    'author': 'PabloLec',
    'author_email': 'pablo.lecolinet@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/PabloLec/recoverpy',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import py_cui

from subprocess import check_output, call
from os import geteuid

from recoverpy import logger as LOGGER


def is_user_root(window: py_cui.PyCUI) -> bool:
    """Checks if user has root privileges.
    The method is simply verifying if EUID == 0.
    It may be problematic in some edge cases. (Some particular OS)
    But, as grep search will not exit quickly, exception handling
    can't be used.

    Args:
        window (py_cui.PyCUI): PyCUI window to display popup.

    Returns:
        bool: User is root
    """

    if geteuid() == 0:
        LOGGER.write("info", "User is root")
        return True

    window.show_error_popup("Not root", "You have to be root or use sudo.")
    LOGGER.write("warning", "User is not root")
    return False


def lsblk() -> list:
    """Uses 'lsblk' utility to generate a list of detected
    system partions."

    Returns:
        list: List of system partitions.
    """

    lsblk_output = check_output(["lsblk", "-r", "-n", "-o", "NAME,TYPE,FSTYPE,MOUNTPOINT"], encoding="utf-8")
    partitions_list_raw = [
        line.strip() for line in lsblk_output.splitlines() if " loop " not in line and "swap" not in line
    ]
    partitions_list_formatted = [line.split(" ") for line in partitions_list_raw]

    LOGGER.write(
        "debug",
        str(partitions_list_formatted),
    )

    return partitions_list_formatted


def format_partitions_list(window: py_cui.PyCUI, raw_lsblk: list) -> dict:
    """Uses lsblk command to find partitions.

    Args:
        window (py_cui.PyCUI): PyCUI window to display popup.
        raw_lsblk (list): Raw lsblk output.

    Returns:
        dict: Found partitions with format :
            {Name: FSTYPE, IS_MOUNTED, MOUNT_POINT}
    """

    # Create dict with relevant infos
    partitions_dict = {}
    for partition in raw_lsblk:
        if len(partition) < 3:
            # Ignore if no FSTYPE detected
            continue

        if len(partition) < 4:
            is_mounted = False
            mount_point = None
        else:
            is_mounted = True
            mount_point = partition[3]

        partitions_dict[partition[0]] = {
            "FSTYPE": partition[2],
            "IS_MOUNTED": is_mounted,
            "MOUNT_POINT": mount_point,
        }

    # Warn the user if no partition found with lsblk
    if len(partitions_dict) == 0:
        LOGGER.write("Error", "No partition found !")
        window.show_error_popup("Error", "No partition found.")
        return None

    LOGGER.write("debug", "Partition list generated using 'lsblk'")
    LOGGER.write(
        "debug",
        f"{str(len(partitions_dict))} partitions found",
    )

    return partitions_dict


def is_progress_installed() -> bool:
    """Verifies if 'progress' tool is installed on current system.

    Returns:
        bool: 'progress' is installed.
    """

    output = call("command -v progress", shell=True)
    if output == 0:
        return True
    else:
        return False
